# This file is autogenerated by the command `make fix-copies`, do not edit.
from ..file_utils import requires_backends


class ImageFeatureExtractionMixin:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class BeitFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class CLIPFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class CLIPProcessor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["vision"])


class DeiTFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class DetrFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class ImageGPTFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class LayoutLMv2FeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class LayoutLMv2Processor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["vision"])


class LayoutXLMProcessor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])

    @classmethod
    def from_pretrained(cls, *args, **kwargs):
        requires_backends(cls, ["vision"])


class PerceiverFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class SegformerFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])


class ViTFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["vision"])
