# This file is autogenerated by the command `make fix-copies`, do not edit.
from ..file_utils import requires_backends


class Speech2TextFeatureExtractor:
    def __init__(self, *args, **kwargs):
        requires_backends(self, ["speech"])
